package de.ozgcloud.archive.bescheid;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.document.bescheid.BescheidServiceGrpc.BescheidServiceBlockingStub;
import de.ozgcloud.document.bescheid.GrpcGetAllBescheidRequest;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service(ArchiveManagerConfiguration.BESCHEID_REMOTE_SERVICE_NAME) // NOSONAR
@RequiredArgsConstructor
class BescheidRemoteService {

	@GrpcClient(ArchiveManagerConfiguration.GRPC_VORGANG_MANAGER_CLIENT_NAME)
	private final BescheidServiceBlockingStub grpcService;
	@Qualifier(ArchiveManagerConfiguration.CALL_CONTEXT_PROVIDER_NAME) // NOSONAR
	private final OzgCloudCallContextProvider contextProvider;
	@Qualifier(ArchiveManagerConfiguration.BESCHEID_MAPPER_NAME) // NOSONAR
	private final BescheidMapper bescheidMapper;

	public Stream<Bescheid> findByVorgangId(String vorgangId) {
		var request = buildGetAllBescheidRequest(vorgangId);

		var response = getGrpcServiceWithInterceptor().getAll(request);

		return response.getBescheidList().stream().map(grpcBescheid -> bescheidMapper.fromGrpc(grpcBescheid, vorgangId));
	}

	GrpcGetAllBescheidRequest buildGetAllBescheidRequest(String vorgangId) {
		return GrpcGetAllBescheidRequest.newBuilder().setVorgangId(vorgangId).build();
	}

	BescheidServiceBlockingStub getGrpcServiceWithInterceptor() {
		return grpcService.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}
}
