package de.ozgcloud.archive.common;

import static java.util.Objects.*;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.Optional;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class DateConverter {

	private static final DateTimeFormatter DATE_TIME_FORMATTER = DateTimeFormatter.ofPattern("dd.MM.yyyy", Locale.GERMAN);

	public static Optional<XMLGregorianCalendar> convertGermanFormatToISO(String dateStr) {
		try {
			if (nonNull(dateStr)) {
				return Optional.of(createDatatypeFactory().newXMLGregorianCalendar(LocalDate.parse(dateStr, DATE_TIME_FORMATTER).toString()));
			}
		} catch (Exception e) {
			LOG.warn("Date '{}' cannot be converted to ISO format.", dateStr, e);
		}
		return Optional.empty();
	}

	public static XMLGregorianCalendar toXmlGregorianCalendar(ZonedDateTime date) {
		try {
			if (nonNull(date)) {
				return createDatatypeFactory().newXMLGregorianCalendar(GregorianCalendar.from(date));
			}
		} catch (Exception e) {
			LOG.warn("Date '{}' cannot be converted to ISO format.", date, e);
		}
		return null;
	}

	public static XMLGregorianCalendar createEmpty() {
		return DateConverter.createDatatypeFactory().newXMLGregorianCalendar();
	}

	static DatatypeFactory createDatatypeFactory() {
		try {
			return DatatypeFactory.newInstance();
		} catch (DatatypeConfigurationException e) {
			throw new TechnicalException("Error creating datatype factory.", e);
		}
	}
}
