package de.ozgcloud.archive.common;

import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class UUIDConverter {

	private static final Pattern UUID_SPLIT_PATTERN = Pattern.compile(
			"^([a-fA-F0-9]{8})([a-fA-F0-9]{4})([a-fA-F0-9]{4})([a-fA-F0-9]{4})([a-fA-F0-9]+)$");

	public static String fromObjectId(String objectId) {
		var matcher = UUID_SPLIT_PATTERN.matcher(objectId);
		if (matcher.find()) {
			var lastGroup = StringUtils.leftPad(matcher.group(5), 12, "0");
			return String.format("%s-%s-%s-%s-%s", matcher.group(1), matcher.group(2), matcher.group(3), matcher.group(4), lastGroup);
		}
		throw new IllegalArgumentException(String.format("Invalid object id %s", objectId));
	}

}
