package de.ozgcloud.archive.file;

import java.io.OutputStream;
import java.util.List;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.archive.common.binaryfile.BinaryFileService;
import de.ozgcloud.archive.vorgang.VorgangWithEingang;
import de.ozgcloud.common.binaryfile.FileId;
import de.xoev.xdomea.DokumentType;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
public class ExportFileService {

	private final OzgFileRemoteService ozgFileRemoteService;
	private final BinaryFileService binaryFileService;

	public Stream<OzgFile> getRepresentations(VorgangWithEingang vorgang) {
		return ozgFileRemoteService.getRepresentations(vorgang.getId());
	}

	public Stream<OzgFile> getAttachments(VorgangWithEingang vorgang) {
		return ozgFileRemoteService.getAttachments(vorgang.getId());
	}

	public Stream<DokumentType> createDokumentTypes(List<OzgFile> ozgFiles, String formEngineName) {
		var ordinalNumberGenerator = new AtomicLong(1);
		return ozgFiles.stream().map(ozgFile -> DokumentTypeBuilder.builder()
				.withOrdinalNumber(ordinalNumberGenerator.getAndIncrement())
				.withFormEngineName(formEngineName)
				.withOzgFile(ozgFile)
				.build());
	}

	public void writeOzgFile(FileId fileId, OutputStream outputStream) {
		binaryFileService.writeFileContent(fileId, outputStream);
	}

}
