package de.ozgcloud.archive.historie;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.archive.common.command.ArchiveManagerCommand;
import de.ozgcloud.archive.common.command.CommandWithPrevious;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class AssignedUserChangeHistoryBuilder extends ChangeHistoryBuilder<AssignedUserChangeHistoryBuilder> {

	static final String BODY_PROPERTY_ASSIGNED_USER = "assignedTo";
	static final String ASSIGN_USER_ORDER = "ASSIGN_USER";

	private UserProfileCache userProfileCache;

	public static AssignedUserChangeHistoryBuilder builder() {
		return new AssignedUserChangeHistoryBuilder();
	}

	public AssignedUserChangeHistoryBuilder withUserProfileCache(UserProfileCache userProfileCache) {
		this.userProfileCache = userProfileCache;
		return this;
	}

	@Override
	boolean isRelevant(ArchiveManagerCommand command) {
		return ASSIGN_USER_ORDER.equals(command.getOrder());
	}

	@Override
	CommandWithChangeValues toCommandWithChangeValues(CommandWithPrevious commandWithPrevious) {
		return new CommandWithChangeValues(
				commandWithPrevious.getCommand(),
				getAssignedUserBeforeChange(commandWithPrevious),
				getAssignedUserAfterChange(commandWithPrevious));
	}

	String getAssignedUserBeforeChange(CommandWithPrevious commandWithPrevious) {
		return Optional.ofNullable(commandWithPrevious.getPrevious())
				.map(this::getAssignedUserFullNameFromCommand)
				.orElse(StringUtils.EMPTY);
	}

	String getAssignedUserAfterChange(CommandWithPrevious commandWithPrevious) {
		return getAssignedUserFullNameFromCommand(commandWithPrevious.getCommand());
	}

	String getAssignedUserFullNameFromCommand(ArchiveManagerCommand command) {
		var assignedUserId = getValueFromCommandBody(BODY_PROPERTY_ASSIGNED_USER, command);
		return userProfileCache.getUserProfile(assignedUserId).getFullName();
	}
}
