package de.ozgcloud.archive.historie;

import java.util.List;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.common.command.ArchiveManagerCommand;
import de.ozgcloud.archive.common.command.CommandService;
import de.ozgcloud.archive.common.user.UserService;
import de.ozgcloud.archive.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
class VorgangChangeHistoryService {

	@Qualifier(ArchiveManagerConfiguration.USER_SERVICE_NAME) // NOSONAR
	private final UserService userService;

	@Qualifier(ArchiveManagerConfiguration.COMMAND_SERVICE_NAME) // NOSONAR
	private final CommandService commandService;

	public VorgangChangeHistory createVorgangChangeHistory(VorgangWithEingang vorgang) {
		var commands = commandService.findFinishedCommands(vorgang.getId()).toList();
		return VorgangChangeHistory.builder()
				.statusChangeHistory(createStatusChangeHistory(vorgang, commands).toList())
				.aktenzeichenChangeHistory(createAktenzeichenChangeHistory(vorgang, commands).toList())
				.assignedUserChangeHistory(createAssignedUserChangeHistory(vorgang, commands).toList())
				.build();
	}

	Stream<VorgangChange> createStatusChangeHistory(VorgangWithEingang vorgang, List<ArchiveManagerCommand> commands) {
		return StatusChangeHistoryBuilder.builder()
				.withCommands(commands)
				.withOrganisationseinheitenID(vorgang.getOrganisationseinheitenID())
				.build();
	}

	Stream<VorgangChange> createAktenzeichenChangeHistory(VorgangWithEingang vorgang, List<ArchiveManagerCommand> commands) {
		return AktenzeichenChangeHistoryBuilder.builder()
				.withCommands(commands)
				.withOrganisationseinheitenID(vorgang.getOrganisationseinheitenID())
				.build();
	}

	Stream<VorgangChange> createAssignedUserChangeHistory(VorgangWithEingang vorgang, List<ArchiveManagerCommand> commands) {
		return AssignedUserChangeHistoryBuilder.builder()
				.withCommands(commands)
				.withOrganisationseinheitenID(vorgang.getOrganisationseinheitenID())
				.withUserProfileCache(UserProfileCache.create(userService::getById))
				.build();
	}

}
