package de.ozgcloud.archive.kommentar;

import java.util.List;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.common.binaryfile.BinaryFileService;
import de.ozgcloud.archive.common.user.UserService;
import de.ozgcloud.archive.file.OzgFile;
import de.ozgcloud.archive.kommentar.KommentarsExportData.KommentarsExportDataBuilder;
import de.ozgcloud.archive.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service(ArchiveManagerConfiguration.KOMMENTAR_SERVICE_NAME)
public class ExportKommentarService {

	@Qualifier(ArchiveManagerConfiguration.KOMMENTAR_REMOTE_SERVICE_NAME) // NOSONAR
	private final KommentarRemoteService kommentarRemoteService;
	@Qualifier(ArchiveManagerConfiguration.BINARY_FILE_SERVICE_NAME) // NOSONAR
	private final BinaryFileService binaryFileService;
	@Qualifier(ArchiveManagerConfiguration.USER_SERVICE_NAME) // NOSONAR
	private final UserService userService;

	public KommentarsExportData createExportData(VorgangWithEingang vorgang) {
		KommentarsExportDataBuilder builder = KommentarsExportData.builder();
		getKommentare(vorgang)
				.map(kommentar -> createKommentarExportData(vorgang, kommentar))
				.forEach(kommentarExportData -> addKommentarExportData(kommentarExportData, builder));
		return builder.build();
	}

	KommentarExportData createKommentarExportData(VorgangWithEingang vorgang, Kommentar kommentar) {
		return new KommentarExportData(vorgang.getOrganisationseinheitenID(), kommentar);
	}

	void addKommentarExportData(KommentarExportData kommentarExportData, KommentarsExportDataBuilder builder) {
		List<OzgFile> attachments = getAttachments(kommentarExportData.kommentar);

		var dokumentType = DokumentTypeBuilder.builder()
				.withKommentarAttachments(attachments)
				.withKommentar(kommentarExportData.kommentar)
				.withOrganisationseinheitenID(kommentarExportData.organisationsEinheitenID)
				.withAuthorFullName(getAuthorFullName(kommentarExportData.kommentar))
				.build();

		builder.dokumentType(dokumentType);
		builder.attachments(attachments);
	}

	List<OzgFile> getAttachments(Kommentar kommentar) {
		return binaryFileService.getFiles(kommentar.getAttachments()).toList();
	}

	String getAuthorFullName(Kommentar kommentar) {
		return userService.getById(kommentar.getCreatedBy()).getFullName();
	}

	Stream<Kommentar> getKommentare(VorgangWithEingang vorgang) {
		return kommentarRemoteService.findByVorgangId(vorgang.getId());
	}

	record KommentarExportData(
			String organisationsEinheitenID,
			Kommentar kommentar) {
	}
}
