package de.ozgcloud.archive.vorgang;

import static java.util.Objects.*;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.util.Collection;

import org.mapstruct.Mapper;

import de.xoev.xdomea.DatatypeType;

@Mapper
interface DatatypeMapper {

	default DatatypeType from(Object object) {
		if (isNull(object)) {
			return DatatypeType.STRING;
		}
		var objectClass = object.getClass();
		if (String.class.equals(objectClass)) {
			return DatatypeType.STRING;
		}
		if (Integer.class.equals(objectClass)) {
			return DatatypeType.INTEGER;
		}
		if (Boolean.class.equals(objectClass)) {
			return DatatypeType.BOOLEAN;
		}
		if (LocalDate.class.equals(objectClass)) {
			return DatatypeType.DATE;
		}
		if (ZonedDateTime.class.equals(objectClass)) {
			return DatatypeType.DATETIME;
		}
		if (Float.class.equals(objectClass)) {
			return DatatypeType.FLOAT;
		}
		if (object instanceof Collection<?> collection && !collection.isEmpty()) {
			return from(collection.iterator().next());
		}
		return DatatypeType.STRING;
	}
}
