/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.apilib.common.command.grpc.GrpcCommandTestFactory;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;

class CommandMapperTest {

	@InjectMocks
	private CommandMapper mapper = Mappers.getMapper(CommandMapper.class);
	@Mock
	private GrpcObjectMapper grpcObjectMapper;

	@Nested
	class TestFromGrpc {

		private GrpcCommand command = GrpcCommandTestFactory.createBuilder()
				.setFinishedAt(CommandTestFactory.FINISHED_AT.toString())
				.setCreatedByName(CommandTestFactory.CREATED_BY_NAME)
				.setOrderString(CommandTestFactory.ORDER)
				.build();

		@DisplayName("on existing body object at command")
		@Nested
		class TestOnExistingBodyObject {

			@Test
			void shouldCallGrpcObjectMapper() {
				mapper.fromGrpc(command);

				verify(grpcObjectMapper).mapFromGrpc(command.getBodyObj());
			}

			@Test
			void shouldMapToCommand() {
				when(grpcObjectMapper.mapFromGrpc(any())).thenReturn(CommandTestFactory.BODY);

				var result = mapper.fromGrpc(command);

				assertThat(result).usingRecursiveComparison().isEqualTo(CommandTestFactory.create());
			}
		}

	}

}