package de.ozgcloud.archive.common.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.apilib.user.GrpcOzgCloudUserProfileService;
import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.user.OzgCloudUserProfile;
import de.ozgcloud.apilib.user.OzgCloudUserProfileTestFactory;

class UserServiceTest {

	@InjectMocks
	private UserService service;

	@Mock
	private GrpcOzgCloudUserProfileService grpcOzgCloudUserProfileService;

	@Mock
	private UserProfileMapper mapper;

	@Nested
	class TestGetById {

		private final String userId = UUID.randomUUID().toString();
		private final OzgCloudUserProfile ozgCloudUserProfile = OzgCloudUserProfileTestFactory.create();

		@Test
		void shouldCallGrpcOzgCloudUserProfileService() {
			callService();

			verify(grpcOzgCloudUserProfileService).getById(OzgCloudUserId.from(userId));
		}

		@Test
		void shouldMap() {
			when(grpcOzgCloudUserProfileService.getById(OzgCloudUserId.from(userId))).thenReturn(ozgCloudUserProfile);

			callService();

			verify(mapper).fromOzgUserProfile(ozgCloudUserProfile);
		}

		@Test
		void shouldRetrunMappedUserProfile() {
			var mappedUserProfile = UserProfileTestFactory.create();
			when(grpcOzgCloudUserProfileService.getById(OzgCloudUserId.from(userId))).thenReturn(ozgCloudUserProfile);
			when(mapper.fromOzgUserProfile(ozgCloudUserProfile)).thenReturn(mappedUserProfile);

			var userProfile = callService();

			assertThat(userProfile).isEqualTo(mappedUserProfile);
		}

		private UserProfile callService() {
			return service.getById(userId);
		}
	}
}
