package de.ozgcloud.archive.vorgang;

import static de.ozgcloud.archive.vorgang.AntragsdatenItemTypeTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.common.test.ITCase;
import de.xoev.xdomea.Antragsdaten;
import de.xoev.xdomea.AntragsdatenGroupType;
import de.xoev.xdomea.AntragsdatenItemType;
import de.xoev.xdomea.AntragsdatenSingleValueFieldType;

@ITCase
class VorgangServiceITCase {

	@Autowired
	private VorgangService vorgangService;

	@Nested
	class TestMapVorgang {

		@Nested
		class TestMapAntragsdaten {

			private static final Map<String, Object> FORM_DATA = Map.of(
					KEY_STRING, VALUE_STRING

			);

			private static final Map<String, Object> OTHER_DATA = Map.of(
					KEY_MAP, Map.of(KEY_COLLECTION, List.of(VALUE_INTEGER)));

			@Test
			void shouldContainSingleAntragsdatenElement() {
				var vorgangType = vorgangService.createVorgangType(VorgangWithEingangTestFactory.create());

				assertThat(vorgangType.getAnwendungsspezifischeErweiterungXML().getAny()).hasSize(1).first().isInstanceOf(Antragsdaten.class);
			}

			@Test
			void shouldMapSingleField() {
				var antragsdaten = mapToAntragsdaten();

				assertThat(antragsdaten.getItem()).hasSize(2);
				assertContainsSingleValueField(antragsdaten.getItem());
			}

			private void assertContainsSingleValueField(List<AntragsdatenItemType> items) {
				var singleValueField = items.stream().filter(item -> item instanceof AntragsdatenSingleValueFieldType).findFirst();
				assertThat(singleValueField).isPresent().get().usingRecursiveComparison().isEqualTo(createSingleValueField());
			}

			@Test
			void shouldMapGroup() {
				var antragsdaten = mapToAntragsdaten();

				assertThat(antragsdaten.getItem()).hasSize(2);
				assertContainsGroup(antragsdaten.getItem());
			}

			private void assertContainsGroup(List<AntragsdatenItemType> items) {
				var group = items.stream().filter(item -> item instanceof AntragsdatenGroupType).findFirst();
				assertThat(group).isPresent().get().extracting(AntragsdatenItemType::getName).isEqualTo(KEY_MAP);
			}

			@Test
			void shouldMapItemsInGroup() {
				var antragsdaten = mapToAntragsdaten();

				assertThat(antragsdaten.getItem()).hasSize(2);
				assertContainsItemsInGroup(antragsdaten.getItem());
			}

			private void assertContainsItemsInGroup(List<AntragsdatenItemType> items) {
				var groupItem = items.stream().filter(item -> item instanceof AntragsdatenGroupType)
						.map(item -> ((AntragsdatenGroupType) item))
						.map(AntragsdatenGroupType::getItem)
						.flatMap(List::stream).findFirst();
				assertThat(groupItem).isPresent().get().usingRecursiveComparison().isEqualTo(createMultiValueField());

			}

			private Antragsdaten mapToAntragsdaten() {
				var eingang = EingangTestFactory.createBuilder().formData(FORM_DATA)
						.antragsteller(AntragstellerTestFactory.createBuilder().otherData(OTHER_DATA).build())
						.build();
				var vorgangWithEingang = VorgangWithEingangTestFactory.createBuilder().eingang(eingang).build();
				var vorgangType = vorgangService.createVorgangType(vorgangWithEingang);
				return (Antragsdaten) vorgangType.getAnwendungsspezifischeErweiterungXML().getAny().get(0);
			}
		}
	}
}